﻿#region GPL License

//Mediaportal (http://www.team-mediaportal.com/) Plugin - CentralizedDatabase
//Copyright (C) 2015 Max Wimmelbacher
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

#endregion

using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MediaPortal.GUI.Library;
using MediaPortal.Database;
using SQLite.NET;
using WindowPlugins.GUITVSeries;
using System.Globalization;
using Cornerstone.Database.CustomTypes;
using System.Reflection;

namespace CentralizedDatabase
{

  /// <summary>
    /// The class CentralizedDatabase.AdditionalMovingPicturesDatabaseSQLite provides all necessary properties/methods which do the real communication with a moving pictures database.
  /// Objects created from this class are returned by calls to methods of the class AdditionalDatabaseFactory,
  /// </summary>

  class AdditionalMovingPicturesDatabaseSQLite : IDisposable
  {

    #region Declaration

    private Dictionary<string, SQLiteResultCacheItem> mCache;
    private SQLiteClient mDatabase;
    private string mDatabaseFile;

    #endregion

    #region Constructors

    public AdditionalMovingPicturesDatabaseSQLite(string strDatabaseFile)
    {
      mDatabaseFile = strDatabaseFile;
      Open();
    }

    #endregion

    #region Properties

    public string DatabaseName
    {
      get
      {
        return mDatabaseFile;
      }
    }

    #endregion Properties

    #region Public methods

    public void Dispose()
    {
      mDatabase.Close();
      mDatabase = null;
    }

    //UpdateDBUserMovieSettings, set the current watch state
    public int UpdateDBUserMovieSettings(int? intnullID, int? intnullUserID, int? intnullUserRating, int intWatched, int intResumePart, int intResumeTime, byte[] bteResumeData, int intResume_BDTitle)
    {
        string strID = "0";
        string strUserID = "1";
        string strUserRating= "NULL";
        string strSQL;
        try
        {
            //Check if we have the id from the row we want to update
            if (intnullID == null)
            {
                return -1;
            }else{
                strID = intnullID.ToString();
            }

            //Check if we have a userID int value, otherwise use the default value of "1" (multiuser support isnt implemented in MovingPictures)
            if (intnullUserID != null) strUserID = intnullUserID.ToString();

            //Check if we have a UserRating int value, otherwise use the default value of "NULL"
            if (intnullUserRating != null) strUserRating = intnullUserRating.ToString();

            //Convert bteResumeData to CornerStone ByteArray and then to string
            ByteArray ResumeData = new ByteArray(bteResumeData);
            if (ResumeData.ToString() == "")
            {
                strSQL = String.Format("UPDATE user_movie_settings SET user = {0}, user_rating = {1}, watched = {2}, resume_part = {3}, resume_time = {4}, resume_data = NULL, resume_titlebd = {5} WHERE id = '{6}'", strUserID, strUserRating, intWatched.ToString(), intResumePart.ToString(), intResumeTime.ToString(), intResume_BDTitle.ToString(), strID);
            }
            else
            {
                strSQL = String.Format("UPDATE user_movie_settings SET user = {0}, user_rating = {1}, watched = {2}, resume_part = {3}, resume_time = {4}, resume_data = '{5}', resume_titlebd = {6} WHERE id = '{7}'", strUserID, strUserRating, intWatched.ToString(), intResumePart.ToString(), intResumeTime.ToString(), ResumeData.ToString(), intResume_BDTitle.ToString(), strID);
            }

            Log.Debug("CentralizedDatabase: (UpdateDBUserMovieSettings) SQL statement '" + strSQL + "' is going to be executed in database '" + mDatabase.DatabaseName + "'.");
            mDatabase.Execute(strSQL);
        }
        catch (Exception ex)
        {
            Log.Error("CentralizedDatabase: UpdateDBUserMovieSettings: MovingPictures database exception err:{0} stack:{1}", ex.Message, ex.StackTrace);
            return -2;
        }

        //OK
        return 1;
    }

    //InsertDBWatchedHistory, insert an watch history entry into the database
    public int InsertDBWatchedHistory(int? intnullUserID, int? intnullMovieID, DateTime dtDateWatched)
    {
        string strUserID = "1";
        string strMovieID = "0";
        string strSQL;
        try
        {
            //Check if we have a userID int value, otherwise use the default value of "1" (multiuser support isnt implemented in MovingPictures)
            if (intnullUserID != null) strUserID = intnullUserID.ToString();

            //Check if we have a Movie id, if not -> error return
            if (intnullMovieID == null)
            {
                return -1;
            }
            else
            {
                strMovieID = intnullMovieID.ToString();
            }

            strSQL = String.Format("INSERT INTO watched_history VALUES (NULL, {0}, {1}, '{2}Z')", strUserID, strMovieID, TimeZone.CurrentTimeZone.ToUniversalTime(dtDateWatched).ToString("yyyy-MM-dd HH:mm:ss"));
            Log.Debug("CentralizedDatabase: (InsertDBWatchedHistory) SQL statement '" + strSQL + "' is going to be executed in database '" + mDatabase.DatabaseName + "'.");
            mDatabase.Execute(strSQL);

            //insert id of movie and watchhistory in table movie_info__watched_history
            strSQL = String.Format("INSERT INTO movie_info__watched_history VALUES (NULL, {0}, {1})", strMovieID, mDatabase.LastInsertID().ToString());
            Log.Debug("CentralizedDatabase: (InsertDBWatchedHistory) SQL statement '" + strSQL + "' is going to be executed in database '" + mDatabase.DatabaseName + "'.");
            mDatabase.Execute(strSQL);
        }
        catch (Exception ex)
        {
            Log.Error("CentralizedDatabase: InsertDBWatchedHistory: MovingPictures database exception err:{0} stack:{1}", ex.Message, ex.StackTrace);
            return -2;
        }

        //OK
        return 1;
    }

    #endregion

    #region Private methods

    private void Open()
    {
      mCache = new Dictionary<string, SQLiteResultCacheItem>();
      Log.Info("CentralizedDatabase: Opening MovingPictures database " + mDatabaseFile);
      try
      {
        if (mDatabase != null)
        {
          Log.Info("CentralizedDatabase: MovingPictures database already opened.");
          return;
        }
        mDatabase = new SQLiteClient(mDatabaseFile);
        DatabaseUtility.SetPragmas(mDatabase);
        Log.Info("CentralizedDatabase: MovingPictures database opened.");
      }
      catch (Exception ex)
      {
        Log.Error("CentralizedDatabase: MovingPictures database exception err:{0} stack:{1}", ex.Message, ex.StackTrace);
      }
    }

    private bool CheckCache(string strTableName, string strKey, string strField, ref string strValue)
    {
        if (mCache.ContainsKey(strTableName) == true)
        {
            if (mCache[strTableName].Key == strKey && mCache[strTableName].IsDirty == false)
            {
                if (strValue == string.Empty)
                {
                    strValue = (string)DatabaseUtility.Get(mCache[strTableName].ResultSet, 0, strField);
                    return true;
                }
                else if (strValue == (string)DatabaseUtility.Get(mCache[strTableName].ResultSet, 0, strField))
                {
                    return true;
                }
                else
                {
                 mCache[strTableName].IsDirty = true;
                }
            }
        }
        return false;
    }

    private void AddToCache(string strTableName, string strKey, SQLiteResultSet objResultSet)
    {
        if (mCache.ContainsKey(strTableName) == true)
        {
            mCache.Remove(strTableName);
        }
        SQLiteResultCacheItem objCacheItem = new SQLiteResultCacheItem(strKey, objResultSet);
        mCache.Add(strTableName, objCacheItem);
    }

    #endregion

  }
}