﻿#region GPL License

//Mediaportal (http://www.team-mediaportal.com/) Plugin - CentralizedDatabase
//Copyright (C) 2015 Max Wimmelbacher
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

#endregion

using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using MediaPortal.Configuration;
using MediaPortal.Profile;
using CentralizedDatabase.Configuration;
using CentralizedDatabase.Classes;

namespace CentralizedDatabase
{
  public partial class CentralizedDatabaseConfig : Form
  {

    #region Declaration

    private List<int> mValidRows = new List<int>();
    public CentralizedDatabaseConfiguration mConfiguration;

    #endregion

    #region Constructors

    public CentralizedDatabaseConfig()
    {
      InitializeComponent();
      InitializeDataGridView();
      LoadConfiguration();
      LoadFormLanguageStrings();
    }

    #endregion

    #region Private methods

    private void InitializeDataGridView()
    {
      if (this.dgvDatabases.Columns.Count == 0)
      {
        DataGridViewCheckBoxColumn colEnabled = new DataGridViewCheckBoxColumn();
        //colEnabled.Name = "Enabled";
        colEnabled.Name = Translation.Enabled;

        colEnabled.AutoSizeMode = DataGridViewAutoSizeColumnMode.ColumnHeader;
        colEnabled.DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft;
        this.dgvDatabases.Columns.Add(colEnabled);

        DataGridViewCheckBoxColumn colCopyAtStartup = new DataGridViewCheckBoxColumn();
        colCopyAtStartup.Name = Translation.CopyAtStartup;
        colCopyAtStartup.AutoSizeMode = DataGridViewAutoSizeColumnMode.ColumnHeader;
        colCopyAtStartup.DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft;
        this.dgvDatabases.Columns.Add(colCopyAtStartup);

        DataGridViewComboBoxColumn colSyncType = new DataGridViewComboBoxColumn();
        colSyncType.Name = Translation.SyncType;
        colSyncType.AutoSizeMode = DataGridViewAutoSizeColumnMode.None;
        colSyncType.Width = 150;
        colSyncType.Items.Add(EnumUtil.GetDescription(CentralizedDatabase.SyncObjectType.Video_database));
        ArrayList alsLoadedPlugins = Plugins.LoadedPlugins;
        foreach (ItemTag objLoop in alsLoadedPlugins)
        {
            if (objLoop.WindowId == 9811)
            {
                colSyncType.Items.Add(EnumUtil.GetDescription(CentralizedDatabase.SyncObjectType.TVSeries_database));
            }
            if (objLoop.WindowId == 96742)
            {
                colSyncType.Items.Add(EnumUtil.GetDescription(CentralizedDatabase.SyncObjectType.MovingPictures_database));
            }
        }
        colSyncType.Items.Add(EnumUtil.GetDescription(CentralizedDatabase.SyncObjectType.SyncFolder));
        colSyncType.Items.Add(EnumUtil.GetDescription(CentralizedDatabase.SyncObjectType.SyncFolderWithDeletions));
        colSyncType.DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft;
        this.dgvDatabases.Columns.Add(colSyncType);

        DataGridViewButtonColumn colSyncSource = new DataGridViewButtonColumn();
        colSyncSource.Name = Translation.SourceFolderDatabase;
        colSyncSource.AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;
        colSyncSource.DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft;
        this.dgvDatabases.Columns.Add(colSyncSource);

        DataGridViewButtonColumn colSyncDest = new DataGridViewButtonColumn();
        colSyncDest.Name = Translation.DestinationFolder;
        colSyncDest.AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;
        colSyncDest.DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft;
        this.dgvDatabases.Columns.Add(colSyncDest);

        this.dgvDatabases.EditMode = DataGridViewEditMode.EditOnEnter;
        this.dgvDatabases.AllowUserToOrderColumns = false;
        this.dgvDatabases.AllowUserToResizeRows = false;
      }
    }

    private void LoadFormLanguageStrings()
    {
        this.Text = "CentralizedDatabase " + Translation.Configuration + " v" + System.Reflection.Assembly.GetExecutingAssembly().GetName().Version.ToString();
        lblDescription.Text = Translation.ConfigurationHint1;
        btnSave.Text = Translation.Save;
        btnCancel.Text = Translation.Cancel;
    }

    private void LoadConfiguration()
    {
      MPSettings objSettings = MPSettings.Instance;
      CentralizedDatabase.SyncObjectType sObject;
      string strConfiguration = objSettings.GetValueAsString("CentralizedDatabase", "configuration", string.Empty);
      if (string.IsNullOrEmpty(strConfiguration))
      {
        this.mConfiguration = new CentralizedDatabaseConfiguration();
      }
      else
      {
        this.mConfiguration = Utils.Deserialize(strConfiguration);
        DataGridViewComboBoxColumn colSyncType = this.dgvDatabases.Columns[Translation.SyncType] as DataGridViewComboBoxColumn;
        foreach (CentralizedDatabaseConfigurationEntry objLoop in this.mConfiguration.mConfigurationEntries)
        {

            //try to get the correct enum value
            sObject = EnumUtil.ParseEnum(objLoop.strDatabaseType, CentralizedDatabase.SyncObjectType.Unknown);
            //backwards compatibility
            if (sObject == CentralizedDatabase.SyncObjectType.Unknown) sObject = EnumUtil.GetValueFromDescription<CentralizedDatabase.SyncObjectType>(objLoop.strDatabaseType);

            if (sObject != CentralizedDatabase.SyncObjectType.Unknown)
            {
                this.dgvDatabases.Rows.Add(objLoop.mEnabled, objLoop.mCopyAtStartup, EnumUtil.GetDescription(sObject), objLoop.strSourceFilePath, objLoop.strDestFilePath);
            }
        }
      }   
    }

    private void ValidateDataGridView()
    {
      this.mValidRows.Clear();
      foreach (DataGridViewRow objLoop in this.dgvDatabases.Rows)
      {
        if (!string.IsNullOrEmpty(objLoop.Cells[1].FormattedValue.ToString()) && !string.IsNullOrEmpty(objLoop.Cells[2].FormattedValue.ToString()))
          this.mValidRows.Add(objLoop.Index);
      }
    }

    private void ValidateDataGridView(int intColumnIndex, int intRowIndex, bool bolKeepState = false)
    {
      if (string.IsNullOrEmpty(this.dgvDatabases[1, intRowIndex].FormattedValue.ToString()) || string.IsNullOrEmpty(this.dgvDatabases[2, intRowIndex].FormattedValue.ToString()))
      {
        if (bolKeepState == false)
        {
          this.dgvDatabases.AllowUserToAddRows = false;
        }
      }
      else
      {
        if (this.mValidRows.Contains(intRowIndex))
        {
          this.mValidRows.Add(intRowIndex);
        }
        if (this.dgvDatabases.Rows.Count - 1 == intRowIndex)
        {
          this.dgvDatabases.AllowUserToAddRows = true;
        }
        else
        {
          if (this.dgvDatabases.AllowUserToAddRows == false)
          {
            ValidateDataGridView(intColumnIndex, this.dgvDatabases.Rows.Count - 1);
          }
        }
      }
    }

    private void DisplayFileDialog(int intColumnIndex, int intRowIndex)
    {
      OpenFileDialog dlgFileBrowser = new OpenFileDialog();
      dlgFileBrowser.CheckPathExists = true;
      dlgFileBrowser.CheckFileExists = true;
      dlgFileBrowser.Multiselect = false;
      dlgFileBrowser.Title = Translation.DataBaseFile;
      if (File.Exists((string)this.dgvDatabases[intColumnIndex, intRowIndex].Value))
      {
        dlgFileBrowser.FileName = this.dgvDatabases[intColumnIndex, intRowIndex].Value.ToString();
      }
      DialogResult resDialogResult = dlgFileBrowser.ShowDialog();
      if (resDialogResult == DialogResult.OK)
      {
        if (ValidateDatabaseFile(dlgFileBrowser.FileName) == true)
        {
          this.dgvDatabases[intColumnIndex, intRowIndex].Value = dlgFileBrowser.FileName;
          this.dgvDatabases.CommitEdit(DataGridViewDataErrorContexts.Commit);
        }
        else
        {
          MessageBox.Show(Translation.NoSQLiteFile, Translation.InvalidSelection, MessageBoxButtons.OK, MessageBoxIcon.Warning);
          DisplayFileDialog(intColumnIndex, intRowIndex);
        }
      }
    }

    private void DisplayFolderDialog(int intColumnIndex, int intRowIndex, string strText)
    {
        FolderBrowserDialog dlgFolderBrowser = new FolderBrowserDialog();
        dlgFolderBrowser.Description = strText;
        if (Directory.Exists((string)this.dgvDatabases[intColumnIndex, intRowIndex].Value))
        {
            dlgFolderBrowser.SelectedPath = this.dgvDatabases[intColumnIndex, intRowIndex].Value.ToString();
        }
        DialogResult resDialogResult = dlgFolderBrowser.ShowDialog(this);
        if (resDialogResult == DialogResult.OK)
        {
            this.dgvDatabases[intColumnIndex, intRowIndex].Value = dlgFolderBrowser.SelectedPath;
            this.dgvDatabases.CommitEdit(DataGridViewDataErrorContexts.Commit);
        }
    }

    private bool ValidateDatabaseFile(string strFileName)
    {
      FileStream fstDatabaseFile = new FileStream(strFileName, FileMode.Open, FileAccess.Read, FileShare.ReadWrite);
      if (fstDatabaseFile.Length >= 16)
      {
        byte[] bteDatabaseFileHeader = new byte[16];
        fstDatabaseFile.Read(bteDatabaseFileHeader, 0, 16);
        fstDatabaseFile.Close();
        string strDatabaseFileHeader = UnicodeEncoding.UTF8.GetString(bteDatabaseFileHeader).Trim('\0');
        if (strDatabaseFileHeader == "SQLite format 3")
        {
          return true;
        }
      }
      return false;
    }

    private void SaveConfiguration()
    {
        this.mConfiguration.mConfigurationEntries = new List<CentralizedDatabaseConfigurationEntry>();
        CentralizedDatabase.SyncObjectType sObject;
        foreach (DataGridViewRow objLoop in this.dgvDatabases.Rows)
        {
            if (!string.IsNullOrEmpty(objLoop.Cells[2].FormattedValue.ToString()) || !string.IsNullOrEmpty(objLoop.Cells[3].FormattedValue.ToString()))
            {
                CentralizedDatabaseConfigurationEntry objEntry = new CentralizedDatabaseConfigurationEntry();
                objEntry.mEnabled = (bool)objLoop.Cells[0].FormattedValue;
                objEntry.mCopyAtStartup = (bool)objLoop.Cells[1].FormattedValue;
                //get db enum type
                sObject = EnumUtil.GetValueFromDescription<CentralizedDatabase.SyncObjectType>((string)objLoop.Cells[2].FormattedValue);
                objEntry.strDatabaseType = sObject.ToString();
                objEntry.strSourceFilePath = (string)objLoop.Cells[3].FormattedValue;
                if ((!string.IsNullOrEmpty(objLoop.Cells[4].FormattedValue.ToString())) && ((sObject == CentralizedDatabase.SyncObjectType.SyncFolder) || (sObject == CentralizedDatabase.SyncObjectType.SyncFolderWithDeletions)))
                {
                    objEntry.strDestFilePath = (string)objLoop.Cells[4].FormattedValue;
                    objEntry.mCopyAtStartup = true;
                }else{
                    objEntry.strDestFilePath = "";
                }
                
                this.mConfiguration.mConfigurationEntries.Add(objEntry);
            }
        }
        string strConfiguration = Utils.Serialize(mConfiguration);
        MPSettings objSettings = MPSettings.Instance;
        objSettings.SetValue("CentralizedDatabase", "configuration", strConfiguration);
    }

    #endregion

    #region Events

    private void btnSave_Click(object sender, EventArgs e)
    {
      SaveConfiguration();
      this.Dispose();
    }

    private void btnCancel_Click(object sender, EventArgs e)
    {
      this.Dispose();
    }

    private void dgvDatabases_CellClick(object sender, DataGridViewCellEventArgs e)
    {
      CentralizedDatabase.SyncObjectType sObject;
      if (e.ColumnIndex == -1)
      {
        return;
      }
      if (dgvDatabases.Columns[e.ColumnIndex] is DataGridViewComboBoxColumn)
      {
        ComboBox cmbComboBox = this.dgvDatabases.EditingControl as ComboBox;
        if(cmbComboBox != null) cmbComboBox.DroppedDown = true;
      }
      else if ((this.dgvDatabases.Columns[e.ColumnIndex] is DataGridViewButtonColumn) && (e.RowIndex != -1))
      {
        //GetDB Type
        sObject = EnumUtil.GetValueFromDescription<CentralizedDatabase.SyncObjectType>((string)dgvDatabases.Rows[e.RowIndex].Cells[2].FormattedValue);

        if ((e.ColumnIndex == 3) && ((sObject == CentralizedDatabase.SyncObjectType.MovingPictures_database) || (sObject == CentralizedDatabase.SyncObjectType.TVSeries_database) || (sObject == CentralizedDatabase.SyncObjectType.Video_database)))
        {
            //Source clicked & database file needed
            DisplayFileDialog(e.ColumnIndex, e.RowIndex);
        }
        else if (((e.ColumnIndex == 3) || (e.ColumnIndex == 4)) && ((sObject == CentralizedDatabase.SyncObjectType.SyncFolder) || (sObject == CentralizedDatabase.SyncObjectType.SyncFolderWithDeletions)))
        {
            //Source Dest clicked & folder needed
            if(e.ColumnIndex == 3)
            {
                DisplayFolderDialog(e.ColumnIndex, e.RowIndex, Translation.SourceFolder);
            }else{
                DisplayFolderDialog(e.ColumnIndex, e.RowIndex, Translation.DestinationFolder);
            }
        }
        ValidateDataGridView(e.ColumnIndex, e.RowIndex,true);
      }
    }

    private void dgvDatabases_CurrentCellDirtyStateChanged(object sender, EventArgs e)
    {
      if (this.dgvDatabases.IsCurrentCellDirty == true)
      {
        this.dgvDatabases.CommitEdit(DataGridViewDataErrorContexts.Commit);
        ValidateDataGridView(this.dgvDatabases.CurrentCell.ColumnIndex, this.dgvDatabases.CurrentCell.RowIndex);
      }
    }

    private void dgvDatabases_RowsRemoved(object sender, DataGridViewRowsRemovedEventArgs e)
    {
      ValidateDataGridView();
    }

    #endregion


  }
}
